CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
111 lines
3.4 KiB
Plaintext
111 lines
3.4 KiB
Plaintext
// Copyright (c) 2020-2025 Advanced Micro Devices, Inc. All rights reserved.
|
|
//
|
|
// Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
// of this software and associated documentation files (the "Software"), to deal
|
|
// in the Software without restriction, including without limitation the rights
|
|
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
|
// copies of the Software, and to permit persons to whom the Software is
|
|
// furnished to do so, subject to the following conditions:
|
|
//
|
|
// The above copyright notice and this permission notice shall be included in all
|
|
// copies or substantial portions of the Software.
|
|
//
|
|
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
|
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
|
|
// SOFTWARE.
|
|
|
|
#include <thrust/device_vector.h>
|
|
#include <thrust/extrema.h>
|
|
#include <thrust/functional.h>
|
|
#include <thrust/host_vector.h>
|
|
#include <thrust/random.h>
|
|
#include <thrust/transform_reduce.h>
|
|
|
|
#include "include/host_device.h"
|
|
|
|
// compute minimum and maximum values in a single reduction
|
|
|
|
// minmax_pair stores the minimum and maximum
|
|
// values that have been encountered so far
|
|
template <typename T>
|
|
struct minmax_pair
|
|
{
|
|
T min_val;
|
|
T max_val;
|
|
};
|
|
|
|
// minmax_unary_op is a functor that takes in a value x and
|
|
// returns a minmax_pair whose minimum and maximum values
|
|
// are initialized to x.
|
|
template <typename T>
|
|
struct minmax_unary_op
|
|
{
|
|
__host__ __device__ minmax_pair<T> operator()(const T& x) const
|
|
{
|
|
minmax_pair<T> result;
|
|
result.min_val = x;
|
|
result.max_val = x;
|
|
return result;
|
|
}
|
|
};
|
|
|
|
// minmax_binary_op is a functor that accepts two minmax_pair
|
|
// structs and returns a new minmax_pair whose minimum and
|
|
// maximum values are the min() and max() respectively of
|
|
// the minimums and maximums of the input pairs
|
|
template <typename T>
|
|
struct minmax_binary_op
|
|
{
|
|
__host__ __device__ minmax_pair<T> operator()(const minmax_pair<T>& x, const minmax_pair<T>& y) const
|
|
{
|
|
minmax_pair<T> result;
|
|
result.min_val = thrust::min(x.min_val, y.min_val);
|
|
result.max_val = thrust::max(x.max_val, y.max_val);
|
|
return result;
|
|
}
|
|
};
|
|
|
|
int main(void)
|
|
{
|
|
// input size
|
|
size_t N = 10;
|
|
|
|
// initialize random number generator
|
|
thrust::default_random_engine rng;
|
|
thrust::uniform_int_distribution<int> dist(10, 99);
|
|
|
|
// initialize data on host
|
|
thrust::device_vector<int> data(N);
|
|
for (size_t i = 0; i < data.size(); i++)
|
|
{
|
|
data[i] = dist(rng);
|
|
}
|
|
|
|
// setup arguments
|
|
minmax_unary_op<int> unary_op;
|
|
minmax_binary_op<int> binary_op;
|
|
|
|
// initialize reduction with the first value
|
|
minmax_pair<int> init = unary_op(data[0]);
|
|
|
|
// compute minimum and maximum values
|
|
minmax_pair<int> result = thrust::transform_reduce(data.begin(), data.end(), unary_op, init, binary_op);
|
|
|
|
// print results
|
|
std::cout << "[ ";
|
|
for (size_t i = 0; i < N; i++)
|
|
{
|
|
std::cout << data[i] << " ";
|
|
}
|
|
std::cout << "]" << std::endl;
|
|
|
|
std::cout << "minimum = " << result.min_val << std::endl;
|
|
std::cout << "maximum = " << result.max_val << std::endl;
|
|
|
|
return 0;
|
|
}
|